import os
from trac.wiki.macros import WikiMacroBase
from genshi.builder import Element

class SwfMacro(WikiMacroBase):
    """Embed an swf file in wiki-formatted text.

    The first argument is the swf file specification. The file specification may
    reference attachments or files in three ways:
     * `module:id:file`, where module can be either '''wiki''' or '''ticket''',
       to refer to the attachment named ''file'' of the specified wiki page or
       ticket.
     * `id:file`: same as above, but id is either a ticket shorthand or a Wiki
       page name.
     * `file` to refer to a local attachment named 'file'. This only works from
       within that wiki page or a ticket.

    Also, the file specification may refer to repository files, using the
    `source:file` syntax (`source:file@rev` works also).

    The remaining arguments are the width and height of the swf file

    You can use swf from other page, other ticket or other module.
    {{{
        [[Swf(movie.swf,400,400)]]            # if attached to the current wiki page
        [[Swf(OtherPage:foo.swf,300,300)]]    # if current module is wiki
        [[Swf(base/sub:bar.swf,300,300)]]     # from hierarchical wiki page
        [[Swf(#3:baz.swf,300,300)]]           # if in a ticket, point to #3
        [[Swf(ticket:36:boo.swf,300,300)]]
        [[Swf(source:/images/bee.swf,300,300)]] # straight from the repository!
        [[Swf(htdocs:foo/bar.swf,300,300)]]   # image file in project htdocs dir.
    }}}
                                                                      
    ''Adapted from the ImageMacro inside the trac distribution''
    """

    def expand_macro(self, formatter, name, content):
        # args will be null if the macro is called without parenthesis.
        if not content:
            return ''
        # parse arguments
        # we expect the 1st argument to be a filename (filespec)
        args = content.split(',')
		if len(args) == 0:
            raise Exception("No argument.")
        filespec = args[0]

        # parse filespec argument to get realm and id if contained.
        parts = filespec.split(':')
        context = None
        if len(parts) == 3:                 # realm:id:attachment-filename
            # TODO: consider using attachment-filename:realm:id?
            realm, id, filename = parts
            context = formatter.context(realm, id) \
                      ('attachment', filename)
        elif len(parts) == 2:
            from trac.versioncontrol.web_ui import BrowserModule
            try:
                browser_links = [res[0] for res in
                                 BrowserModule(self.env).get_link_resolvers()]
            except Exception:
                browser_links = []
            if parts[0] in browser_links:   # source:path
                # TODO: use context here as well
                realm, filename = parts
                rev = None
                if '@' in filename:
                    filename, rev = filename.split('@')
                raw_url = formatter.href.browser(filename, rev=rev, format='raw')
                desc = filespec
            else: # #ticket:attachment or WikiPage:attachment
                # FIXME: do something generic about shorthand forms...
                realm = None
                id, filename = parts
                if id and id[0] == '#':
                    realm = 'ticket'
                    id = id[1:]
                elif id == 'htdocs':
                    raw_url = url = formatter.href.chrome('site', filename)
                    desc = os.path.basename(filename)
                elif id in ('http', 'https', 'ftp'): # external URLs
                    raw_url = url = desc = id+':'+filename
                else:
                    realm = 'wiki'
                if realm:
                    context = formatter.context(realm, id) \
                              ('attachment', filename)
                              elif len(parts) == 1:               # attachment
            context = formatter.context('attachment', filespec)
        else:
            raise Exception('No filespec given')
        if context: # this is an attachment
            raw_url = context.resource_href(format='raw')

        # set default width, height if not given
        swfwidth = 300
        swfheight = 300

        if len(args) == 2:
                swfwidth = args[1]
        if len(args) == 3:
                swfwidth = args[1]
                swfheight = args[2]


        result = Element('p')(Element( 'object', type='application/x-shockwave-flash', data=raw_url, width=swfwidth, height=swfheight ))
        return result
